#include <LiquidCrystal_I2C.h> //I2C Character LCD Library include
//you can download the library from here https://github.com/johnrickman/LiquidCrystal_I2C

LiquidCrystal_I2C lcd(0x3F,16,2); //LCD I2C Module address
/*If display not working, then the I2C Module might be different according to the manufacturer
**try connecting the module to the arduino and upload the Scanner example from I2CScanner.h library by Luis Llamas
*download the library from here https://github.com/luisllamasbinaburo/Arduino-I2CScanner
**I used a similiar module to the one in this instructable https://www.instructables.com/LCD-With-I2C/
**If your chip has a PCF8574 Chip from Texas Instruments then the address is probably 0x27
***If your chip has a PCF8574 Chip from NXP then the address is probably 0x3F
*/


#define signalPin 2 //Input signal from the UT373, Must be interrupt capable pin (Pin 2 or 3 on arduino Uno, Nano, ProMini)

unsigned long signalMicros = 0; //micros() count at the pulse detection moment
unsigned long lastMicros = 0; //micros() count for the previous pulse
unsigned long periodicTime = 0; //peridic time of the signal, it is the difference between current pulse micros (=signalMicros) and last pulse micros(=lastmicros)
unsigned int RPMValue = 0; //Caluclated RPM Value (Revolutions Per Second) by dividing 60,000,000 over periodicTime variable
float RPSValue = 0.0; //Calculated RPS Value (Revolutions Per Second) by dividing RPMValue by 60.0

char LCDDataBuffer[32]; //LCD Display data are stored in this buffer
char RPSBuffer[10]; //buffer to convert float RPSValue to string

void UT373_Micros();

void setup() {
Serial.begin(115200);
attachInterrupt(digitalPinToInterrupt(signalPin), UT373_Micros, FALLING);
lcd.init();
lcd.backlight(); 
//lcd.noBacklight();

}

void loop() {
if (micros() - lastMicros > 1000000){ //If no pulses are detected for 1 second (1,000,000 Microseconds)
  lcd.clear();
  lcd.print("NO SIGNAL");
  delay(250);
  Serial.println("NO SIGNAL");
}
else { //if pulses are detected and variables updated from the interrupt function (UT373_Micros())
  RPMValue = 60000000 / periodicTime;
  RPSValue = RPMValue / 60.0 ;
  lcd.setCursor(0,0);
  sprintf(LCDDataBuffer, "% 6d RPM      ", RPMValue); 
  lcd.print(LCDDataBuffer);
  lcd.setCursor(0,1);
  dtostrf(RPSValue, 6, 1, RPSBuffer);
  sprintf(LCDDataBuffer, "%s RPS", RPSBuffer); 
  lcd.print(LCDDataBuffer);
  Serial.println(RPMValue);

  delay(250);
  
}
  

}

void UT373_Micros(){
  signalMicros = micros(); //Current pulse timestamp
  periodicTime = signalMicros - lastMicros; //periodic time calculation by calculating current pulse timestamp - last pulse timestamp
  lastMicros = signalMicros; //saving current pulse time to be used as a previous pulse time for the next pulse
}

